@@ -20,7 +20,9 @@ def generate(args, parser, extra, subparser):
 
     # Json output
     if args.json:
-        result = setup.get_json(corpus, system_libs=args.system_libs)
+        result = setup.get_json(
+            corpus, system_libs=args.system_libs, globals_only=args.globals_only
+        )
         print(json.dumps(result, indent=4))
 
     # Asp output
@@ -28,7 +30,13 @@ def generate(args, parser, extra, subparser):
         # Get output via StringIO
         out = io.StringIO()
         driver = PyclingoDriver(out=out)
-        driver.solve(setup, [corpus], facts_only=True, system_libs=args.system_libs)
+        driver.solve(
+            setup,
+            [corpus],
+            facts_only=True,
+            system_libs=args.system_libs,
+            is_single=True,
+        )
         asp = out.getvalue()
         out.close()
         print(asp)
